"use strict";
cc._RF.push(module, 'a6ea5aOGCFJs54l3JSpA8gB', 'ListViewNew');
// Script/ListViewNew.js

"use strict";

/* jshint esversion:6 */

/**
 * 仅调用一次
 * @param content 錨點需要設置為0.5；
 * @param item 需要繼承ListViewItemNew；
 */
// let ListViewItem = require("ListViewItemNew");
cc.Class({
  "extends": cc.Component,
  properties: {
    scrollView: cc.ScrollView,
    itemPrefab: cc.Prefab,
    //item预制体 预制挂的脚本要继承ListViewItem
    direction: cc.Float = 1,
    //1代表豎屏
    spawnCount: cc.Float = 0,
    // 实际创建的项数量
    spacingY: cc.Float = 0,
    // 纵向间隔
    spacingX: cc.Float = 0,
    // 横向间隔
    itemHeight: cc.Float = 0,
    // item的高度
    itemWidth: cc.Float = 0,
    // item的高度
    colNum: cc.Float = 1,
    // 纵向布局时的列数
    rowNum: cc.Float = 1,
    // 横向布局时的行数
    // 兼容上下拉刷新
    isPullUpdate: {
      "default": false,
      tooltip: "是否打开上拉、下拉刷新"
    },
    topUpdateCallback: {
      "default": null,
      type: cc.Component.EventHandler,
      tooltip: "顶部下拉刷新回调事件，不传默认关闭下拉刷新",
      visible: function visible() {
        return this.isPullUpdate;
      }
    },
    downUpdateCallback: {
      "default": null,
      type: cc.Component.EventHandler,
      tooltip: "底部上拉刷新回调事件，不传默认关闭上拉刷新",
      visible: function visible() {
        return this.isPullUpdate;
      }
    }
  },
  onLoad: function onLoad() {},

  /**
   * 仅调用一次
   * @param list 列表数据
   * @param obj 额外数据
   */
  init: function init(list, obj) {
    if (this.initialled) {
      return;
    }

    this.DirectionVertical = 1; //1為豎屏 其他為橫屏

    this.initialled = false;
    this.obj = obj;
    this.dataList = list || [];
    this.totalCount = this.dataList.length;
    this.itemList = []; // 存储实际创建的项数组

    this.content = this.scrollView.content;
    this.updateTimer = 0;
    this.updateInterval = 0.05;
    this.lastContentPosY = 0; // 使用这个变量来判断滚动操作是向上还是向下

    this.lastContentPosX = 0;
    this.lastContentPosyPageLoad = 0; //这个属性是特殊场景上用，加载更多的最后恢复的位置（在上拉过程中会重置lastContentPosY，所有不准确）
    // 设定缓冲矩形的大小为实际创建项的高度累加，当某项超出缓冲矩形时，则更新该项的显示内容

    this.bufferZoneY = Math.ceil(this.spawnCount / this.colNum) * (this.itemHeight + this.spacingY) / 2; // 设定缓冲矩形的大小为实际创建项的宽度累加，当某项超出缓冲矩形时，则更新该项的显示内容

    this.bufferZoneX = Math.ceil(this.spawnCount / this.rowNum) * (this.itemWidth + this.spacingX) / 2;
    this.initialize();
    this.initialled = true;
    this.callBack = null;
    this.addPullScrollComp();
  },
  // 添加上下拉刷新组件
  addPullScrollComp: function addPullScrollComp() {
    if (this.isPullUpdate) {
      var PullScrollView = require("PullScrollView");

      this.jsPullScroll = this.node.addComponent(PullScrollView);
      this.jsPullScroll.initForListView(this);
    }
  },
  // 高度改变时更新上下拉的高度
  refreshPullUpdate: function refreshPullUpdate() {
    var _this = this;

    if (this.jsPullScroll) {
      if (this.topUpdateCallback && this.topUpdateCallback.target && this.downUpdateCallback && this.downUpdateCallback.target) {
        this.jsPullScroll.refreshView(function () {
          _this.topUpdateCallback.emit();
        }, function () {
          _this.downUpdateCallback.emit();
        });
      } else if (this.topUpdateCallback && this.topUpdateCallback.target) {
        this.jsPullScroll.refreshView(function () {
          _this.topUpdateCallback.emit();
        }, null);
      } else if (this.downUpdateCallback && this.downUpdateCallback.target) {
        this.jsPullScroll.refreshView(null, function () {
          _this.downUpdateCallback.emit();
        });
      }
    }
  },
  removeAllItems: function removeAllItems() {
    for (var key in this.itemList) {
      if (this.itemList[key]) {
        this.itemList[key] = null;
      }
    }

    this.itemList = [];

    if (this.content) {
      this.content.removeAllChildren();
    }
  },
  // 列表初始化
  initialize: function initialize() {
    // 获取整个列表的高度
    if (this.direction == this.DirectionVertical) {
      this.content.height = Math.ceil(this.totalCount / this.colNum) * (this.itemHeight + this.spacingY) + this.spacingY;

      for (var i = 0; i < this.spawnCount; ++i) {
        // spawn items, we only need to do this once
        var item = cc.instantiate(this.itemPrefab);
        this.content.addChild(item);
        item.active = true; //设置该item的坐标（注意父节点content的Anchor坐标是(0.5, 1)，所以item的y坐标总是负值）

        var row = Math.floor(i / this.colNum);
        var col = i % this.colNum;
        var x = (this.itemWidth + this.spacingX) * (-(this.colNum - 1) * 0.5 + col);
        item.setPosition(x, -this.itemHeight * (0.5 + row) - this.spacingY * (row + 1));
        item.getComponent("ListViewItemNew").index = i;
        item.getComponent("ListViewItemNew").updateItem(i, this.dataList[i], this.obj);
        this.itemList.push(item);
      }
    } else {
      this.content.width = Math.ceil(this.totalCount / this.rowNum) * (this.itemWidth + this.spacingX) + this.spacingX;

      for (var _i = 0; _i < this.spawnCount; ++_i) {
        // spawn items, we only need to do this once
        var _item = cc.instantiate(this.itemPrefab);

        this.content.addChild(_item);
        _item.active = true; // 设置该item的坐标（注意父节点content的Anchor坐标是(0, 0.5)，所以item的x坐标总是正值）

        var _col = Math.floor(_i / this.rowNum);

        var _row = _i % this.rowNum;

        var y = (this.itemHeight + this.spacingY) * ((this.rowNum - 1) * 0.5 - _row);

        _item.setPosition(this.itemWidth * (0.5 + _col) + this.spacingX * (_col + 1), y);

        _item.getComponent("ListViewItemNew").index = _i;

        _item.getComponent("ListViewItemNew").updateItem(_i, this.dataList[_i], this.obj);

        this.itemList.push(_item);
      }
    }
  },
  // 返回item在ScrollView空间的坐标值
  getPositionInView: function getPositionInView(item) {
    if (cc.isValid(item) && cc.isValid(item.parent)) {
      var worldPos = item.parent.convertToWorldSpaceAR(item.position);
      var viewPos = this.scrollView.node.convertToNodeSpaceAR(worldPos);
      return viewPos;
    }
  },
  // 每帧调用一次。根据滚动位置动态更新item的坐标和显示(所以spawnCount可以比totalCount少很多)
  lateUpdate: function lateUpdate() {
    if (!this.initialled) {
      return;
    }

    if (this.updateTimer < this.updateInterval || this.itemList && this.itemList.length <= 0) {
      // logd("updateTimer ========return");
      return; // 我们不需要每帧都做计算
    }

    this.updateTimer = 0;
    var items = this.itemList;

    if (this.direction == this.DirectionVertical) {
      // 如果当前content的y坐标小于上次记录值，则代表往下滚动，否则往上。
      var isDown = this.content.y < this.lastContentPosY; // 实际创建项占了多高（即它们的高度累加）

      var offset = (this.itemHeight + this.spacingY) * Math.ceil(this.spawnCount / this.colNum);
      var newY = 0; // 遍历数组，更新item的位置和显示

      for (var i = 0; i < items.length; ++i) {
        var viewPos = this.getPositionInView(items[i]);

        if (isDown) {
          // 提前计算出该item的新的y坐标
          newY = items[i].y + offset; // 如果往下滚动时item已经超出缓冲矩形，且newY未超出content上边界，
          // 则更新item的坐标（即上移了一个offset的位置），同时更新item的显示内容

          if (viewPos.y < -this.bufferZoneY && newY < 0) {
            items[i].y = newY;
            var itemComp = items[i].getComponent("ListViewItemNew");
            var index = itemComp.index - items.length; // 更新index及数据下标

            itemComp.index = index; // logd('listView ====================0 : itemComp.index ：' + index);

            itemComp.updateItem(index, this.dataList[index], this.obj);
          }
        } else {
          // 提前计算出该item的新的y坐标
          newY = items[i].y - offset; // 如果往上滚动时item已经超出缓冲矩形，且newY未超出content下边界，
          // 则更新item的坐标（即下移了一个offset的位置），同时更新item的显示内容

          if (viewPos.y > this.bufferZoneY && newY > -this.content.height) {
            items[i].y = newY;

            var _itemComp = items[i].getComponent("ListViewItemNew");

            var _index = _itemComp.index + items.length; // 更新index及数据下标


            _itemComp.index = _index; // logd('listView ====================1 : itemComp.index ：' + index + " newY: "+newY);

            _itemComp.updateItem(_index, this.dataList[_index], this.obj);

            this.lastContentPosyPageLoad = this.content.y;
          }
        }
      } // 更新lastContentPosY和总项数显示


      this.lastContentPosY = this.content.y;
    } else {
      // 如果当前content的x坐标小于上次记录值，则代表往左滚动，否则往右。
      var isLeft = this.content.x < this.lastContentPosX; // 实际创建项占了多高（即它们的宽度累加）

      var _offset = (this.itemWidth + this.spacingX) * Math.ceil(this.spawnCount / this.rowNum);

      var newX = 0; // 遍历数组，更新item的位置和显示

      for (var _i2 = 0; _i2 < items.length; ++_i2) {
        var _viewPos = this.getPositionInView(items[_i2]);

        if (isLeft) {
          // 提前计算出该item的新的y坐标
          newX = items[_i2].x + _offset; // 如果往左滚动时item已经超出缓冲矩形，且newX未超出content右边界，
          // 则更新item的坐标（即右移了一个offset的位置），同时更新item的显示内容

          if (_viewPos.x < -this.bufferZoneX && newX < this.content.width) {
            items[_i2].x = newX;

            var _itemComp2 = items[_i2].getComponent("ListViewItemNew");

            var _index2 = _itemComp2.index + items.length; // 更新index及数据下标


            _itemComp2.index = _index2;

            _itemComp2.updateItem(_index2, this.dataList[_index2], this.obj);
          }
        } else {
          // 提前计算出该item的新的y坐标
          newX = items[_i2].x - _offset; // 如果往右滚动时item已经超出缓冲矩形，且newY未超出content左边界，
          // 则更新item的坐标（即左移了一个offset的位置），同时更新item的显示内容

          if (_viewPos.x > this.bufferZoneX && newX > 0) {
            items[_i2].x = newX;

            var _itemComp3 = items[_i2].getComponent("ListViewItemNew");

            var _index3 = _itemComp3.index - items.length; // 更新index及数据下标


            _itemComp3.index = _index3;

            _itemComp3.updateItem(_index3, this.dataList[_index3], this.obj);
          }
        }
      } // 更新lastContentPosY和总项数显示


      this.lastContentPosX = this.content.x;
    }
  },
  update: function update(dt) {
    if (!this.initialled) {
      return;
    }

    this.updateTimer += dt;

    if (this.updateTimer < this.updateInterval || this.itemList && this.itemList.length <= 0) {
      // logd("updateTimer ========return");
      return; // 我们不需要每帧都做计算
    }
  },
  //更新列表内数据
  updateData: function updateData(list, isSplitLoad, notScrollToTop) {
    this.dataList = list;
    var NotScrollToTop = this.dataList && this.dataList.length > this.spawnCount;
    this.scrollView.stopAutoScroll();

    if (isSplitLoad) {
      this.updateDataAndSplitLoad(this.dataList);
      return;
    } // 


    if (this.direction == this.DirectionVertical) {
      if (!notScrollToTop || !NotScrollToTop) this.scrollView.scrollToTop();

      for (var i = 0; i < this.itemList.length; i++) {
        var row = Math.floor(i / this.colNum);
        this.itemList[i].y = -this.itemHeight * (0.5 + row) - this.spacingY * (row + 1);
        var itemComp = this.itemList[i].getComponent("ListViewItemNew");
        var data = this.dataList[i];
        itemComp.index = i; // logd('listView content lastContentPosY2：' + this.lastContentPosY);

        itemComp.updateItem(i, data, this.obj);
      }

      this.content.height = Math.ceil(this.dataList.length / this.colNum) * (this.itemHeight + this.spacingY) + this.spacingY;
    } else {
      if (!notScrollToTop || !NotScrollToTop) this.scrollView.scrollToLeft();

      for (var _i3 = 0; _i3 < this.itemList.length; _i3++) {
        var col = Math.floor(_i3 / this.rowNum);
        this.itemList[_i3].x = this.itemWidth * (0.5 + col) + this.spacingX * (col + 1);

        var _itemComp4 = this.itemList[_i3].getComponent("ListViewItemNew");

        var _data = this.dataList[_i3];
        _itemComp4.index = _i3;

        _itemComp4.updateItem(_i3, _data, this.obj);
      }

      this.content.width = Math.ceil(this.dataList.length / this.rowNum) * (this.itemWidth + this.spacingX) + this.spacingX;
    }

    this.updateHeightForData();
  },
  updateDataNoChangeIndex: function updateDataNoChangeIndex(list) {
    if (!list || list.length <= 0) {
      this.dataList.content.x = 0;
      return;
    }

    this.dataList = list;

    for (var i = 0; i < this.itemList.length; i++) {
      var itemComp = this.itemList[i].getComponent("ListViewItemNew");
      var data = this.dataList[itemComp.index];
      itemComp.updateItem(itemComp.index, data, list);
    }

    this.content.width = Math.ceil(this.dataList.length / this.rowNum) * (this.itemWidth + this.spacingX) + this.spacingX;
  },
  // 更新列表内数据,分帧加载。
  updateDataAndSplitLoad: function updateDataAndSplitLoad(list) {
    var _this2 = this;

    this.initialled = false;
    this.dataList = list;
    this.scrollView.stopAutoScroll();

    if (this.callBack) {
      this.unschedule(this.callBack);
    }

    if (this.direction == this.DirectionVertical) {
      var i = 0;
      this.scrollView.scrollToTop();
      this.content.height = 0;

      this.callBack = function () {
        if (i >= _this2.itemList.length || !cc.isValid(_this2.content)) {
          _this2.unschedule(_this2.callBack);

          _this2.callBack = null;
          _this2.initialled = true;
          if (_this2.content) _this2.content.height = Math.ceil(_this2.dataList.length / _this2.colNum) * (_this2.itemHeight + _this2.spacingY) + _this2.spacingY;
          return;
        }

        var row = Math.floor(i / _this2.colNum);
        _this2.itemList[i].y = -_this2.itemHeight * (0.5 + row) - _this2.spacingY * (row + 1);

        var itemComp = _this2.itemList[i].getComponent("ListViewItemNew");

        var data = _this2.dataList[i];
        itemComp.index = i;
        itemComp.updateItem(i, data, _this2.obj);
        i++;
      };
    } else {
      var _i4 = 0;
      this.scrollView.scrollToLeft();
      this.content.width = 0;

      this.callBack = function () {
        if (_i4 >= _this2.itemList.length || !cc.isValid(_this2.content)) {
          _this2.unschedule(_this2.callBack);

          _this2.callBack = null;
          _this2.initialled = true;
          if (_this2.content) _this2.content.width = Math.ceil(_this2.dataList.length / _this2.rowNum) * (_this2.itemWidth + _this2.spacingX) + _this2.spacingX;
          return;
        }

        var col = Math.floor(_i4 / _this2.rowNum);
        _this2.itemList[_i4].x = _this2.itemWidth * (0.5 + col) + _this2.spacingX * (col + 1);

        var itemComp = _this2.itemList[_i4].getComponent("ListViewItemNew");

        itemComp.index = _i4;
        var data = _this2.dataList[_i4];
        itemComp.updateItem(_i4, data, _this2.obj);
        _i4++;
      };
    }

    this.schedule(this.callBack, 0.02, cc.macro.REPEAT_FOREVER);
  },
  updateHeightForData: function updateHeightForData(data) {
    if (data) {
      this.dataList = data;
    }

    if (this.dataList) {
      var length = this.dataList.length;

      if (this.direction == this.DirectionVertical) {
        // 计算真实高度
        this.content.height = Math.ceil(length / this.colNum) * (this.itemHeight + this.spacingY) + this.spacingY; // 设置最小高度，使其能滚动

        var viewHeight = this.content.parent.height;

        if (this.content.height < viewHeight) {
          this.content.height = viewHeight;
        }
      } else {
        // 计算真实宽度
        this.content.width = Math.ceil(length / this.rowNum) * (this.itemWidth + this.spacingX) + this.spacingX; // 设置最小宽度，使其能滚动

        var viewWidth = this.content.parent.width;

        if (this.content.width < viewWidth) {
          this.content.width = viewWidth;
        }
      }
    }

    this.refreshPullUpdate();
  },
  // 纵向跳转到对应toIndex下标item的位置，offsetY必须是item高的倍数
  listToOffsetY: function listToOffsetY(toIndex, offsetY) {
    if (!toIndex || toIndex == 0) {
      return;
    }

    var scrollOffset = this.scrollView.getScrollOffset();
    this.scrollView.scrollToOffset(cc.v2(scrollOffset.x, offsetY)); // logd('>>> listScrollToOffset:', this.content.y,'  lastContentPosY:',this.lastContentPosY);

    this.lastContentPosY = this.content.y; // 遍历数组，更新item的位置和显示

    var items = this.itemList;

    for (var i = 0; i < items.length; ++i) {
      items[i].y = -offsetY - this.itemHeight * (0.5 + i);
      var itemComp = items[i].getComponent("ListViewItemNew");
      var index = toIndex + i;
      itemComp.index = index;
      itemComp.updateItem(index, this.dataList[index], this.obj);
    }
  },
  onDestroy: function onDestroy() {//clearInterval(this.IntervalId);
  }
});

cc._RF.pop();